@@ -198,7 +198,6 @@ Debug information is inserted in the buffer \"*IMAP4 DEBUG*\"")
 	    (insert "NO LOGGING\n")
 	  (insert (apply 'format message args) "\n")))))
 
-
 (defsubst elmo-imap4-decode-folder-string (string)
   (if elmo-imap4-use-modified-utf7
       (utf7-decode-string string 'imap)
@@ -284,7 +283,7 @@ Returns a TAG string which is assigned to the COMMAND."
 	(elmo-imap4-process-bye session))
       (setq elmo-imap4-current-response nil)
       (if elmo-imap4-parsing
-	  (error "IMAP process is running. Please wait (or plug again.)"))
+	  (error "IMAP process is running. Please wait (or plug again)"))
       (setq elmo-imap4-parsing t)
       (elmo-imap4-debug "<-(%s)- %s" tag command)
       (while (setq token (car command-args))
@@ -640,7 +639,12 @@ BUFFER must be a single-byte buffer."
 		    (elmo-imap4-folder-mailbox-internal folder)))))
 
 (defun elmo-imap4-get-session (folder &optional if-exists)
-  (elmo-network-get-session 'elmo-imap4-session "IMAP" folder if-exists))
+  (elmo-network-get-session 'elmo-imap4-session
+			    (concat
+			     (if (elmo-folder-biff-internal folder)
+				 "BIFF-")
+			     "IMAP")
+			    folder if-exists))
 
 (defun elmo-imap4-session-select-mailbox (session mailbox
 						  &optional force no-error)
@@ -1969,7 +1973,7 @@ Return nil if no complete line has arrived."
 (luna-define-method elmo-folder-writable-p ((folder elmo-imap4-folder))
   t)
 
-(luna-define-method elmo-folder-delete ((folder elmo-imap4-folder))
+(luna-define-method elmo-folder-delete :before ((folder elmo-imap4-folder))
   (let ((session (elmo-imap4-get-session folder))
 	msgs)
     (when (elmo-imap4-folder-mailbox-internal folder)
@@ -2071,7 +2075,7 @@ If optional argument REMOVE is non-nil, remove FLAG."
 	(imap-search-keys '("bcc" "body" "cc" "from" "subject" "to"))
 	(total 0)
 	(length (length from-msgs))
-	charset set-list results)
+	charset set-list end results)
     (message "Searching...")
     (cond
      ((string= "last" search-key)
@@ -2091,8 +2095,9 @@ If optional argument REMOVE is non-nil, remove FLAG."
       (setq search-key (concat "sent" search-key)
 	    set-list (elmo-imap4-make-number-set-list
 		      from-msgs
-		      elmo-imap4-number-set-chop-length))
-      (while set-list
+		      elmo-imap4-number-set-chop-length)
+	    end nil)
+      (while (not end)
 	(setq results
 	      (append
 	       results
@@ -2122,7 +2127,8 @@ If optional argument REMOVE is non-nil, remove FLAG."
 	  (elmo-display-progress
 	   'elmo-imap4-search "Searching..."
 	   (/ (* total 100) length)))
-	(setq set-list (cdr set-list)))
+	(setq set-list (cdr set-list)
+	      end (null set-list)))
       results)
      (t
       (setq charset
@@ -2132,8 +2138,9 @@ If optional argument REMOVE is non-nil, remove FLAG."
 	       (elmo-filter-value filter)))
 	    set-list (elmo-imap4-make-number-set-list
 		      from-msgs
-		      elmo-imap4-number-set-chop-length))
-      (while set-list
+		      elmo-imap4-number-set-chop-length)
+	    end nil)
+      (while (not end)
 	(setq results
 	      (append
 	       results
@@ -2172,7 +2179,8 @@ If optional argument REMOVE is non-nil, remove FLAG."
 	  (elmo-display-progress
 	   'elmo-imap4-search "Searching..."
 	   (/ (* total 100) length)))
-	(setq set-list (cdr set-list)))
+	(setq set-list (cdr set-list)
+	      end (null set-list)))
       results))))
 
 (defun elmo-imap4-search-internal (folder session condition from-msgs)
@@ -2515,14 +2523,15 @@ If optional argument REMOVE is non-nil, remove FLAG."
       (setq elmo-imap4-display-literal-progress nil))
     (unless elmo-inhibit-display-retrieval-progress
       (elmo-display-progress 'elmo-imap4-display-literal-progress
-			     "" 100)  ; remove progress bar.
+			     "Retrieving..." 100)  ; remove progress bar.
       (message "Retrieving...done."))
     (if (setq response (elmo-imap4-response-bodydetail-text
 			(elmo-imap4-response-value-all
 			 response 'fetch)))
 	(with-current-buffer outbuf
 	  (erase-buffer)
-	  (insert response)))))
+	  (insert response)
+	  t))))
 
 (luna-define-method elmo-message-fetch-plugged ((folder elmo-imap4-folder)
 						number strategy
