@@ -295,7 +295,7 @@
 	(elmo-make-directory dir))
       t)))
 
-(luna-define-method elmo-folder-delete ((folder elmo-localdir-folder))
+(luna-define-method elmo-folder-delete :before ((folder elmo-localdir-folder))
   (let ((dir (elmo-localdir-folder-directory-internal folder)))
     (if (not (file-directory-p dir))
 	(error "No such directory: %s" dir)
@@ -332,12 +332,14 @@
     (cond
      ;; short cut.
      ((and (vectorp condition)
+	   (not (eq (elmo-filter-type condition) 'unmatch))
 	   (string= (elmo-filter-key condition) "last"))
       (nthcdr (max (- (length msgs)
 		      (string-to-int (elmo-filter-value condition)))
 		   0)
 	      msgs))
      ((and (vectorp condition)
+	   (not (eq (elmo-filter-type condition) 'unmatch))
 	   (string= (elmo-filter-key condition) "first"))
       (let ((rest (nthcdr (string-to-int (elmo-filter-value condition) )
 			  msgs)))
