@@ -245,7 +245,10 @@ Don't cache if nil.")
 (defun elmo-nntp-get-session (folder &optional if-exists)
   (elmo-network-get-session
    'elmo-nntp-session
-   "NNTP"
+   (concat
+    (if (elmo-folder-biff-internal folder)
+	"BIFF-")
+    "NNTP")
    folder
    if-exists))
 
@@ -260,7 +263,7 @@ Don't cache if nil.")
       (while (and (memq (process-status process) '(open run))
 		  (goto-char (point-max))
 		  (forward-line -1)
-		  (not (looking-at "20[01]")))
+		  (not (looking-at "^[2-5][0-9][0-9]")))
 	(accept-process-output process 1))
       (setq elmo-nntp-read-point (point))
       (or (elmo-nntp-read-response session t)
@@ -334,14 +337,14 @@ Don't cache if nil.")
 	(setq response-string
 	      (buffer-substring elmo-nntp-read-point (- match-end 2)))
 	(goto-char elmo-nntp-read-point)
-	(if (looking-at "[23][0-9]+ .*$")
+	(if (looking-at "[234][0-9]+ .*$")
 	    (progn (setq response-continue nil)
 		   (setq elmo-nntp-read-point match-end)
 		   (setq response
 			 (if response
 			     (concat response "\n" response-string)
 			   response-string)))
-	  (if (looking-at "[^23][0-9]+ .*$")
+	  (if (looking-at "[^234][0-9]+ .*$")
 	      (progn (setq response-continue nil)
 		     (setq elmo-nntp-read-point match-end)
 		     (setq response nil))
@@ -386,7 +389,8 @@ Don't cache if nil.")
       (with-current-buffer outbuf
 	(erase-buffer)
 	(insert-buffer-substring (elmo-network-session-buffer session)
-				 start (- end 3))))))
+				 start (- end 3))))
+    t))
 
 (defun elmo-nntp-select-group (session group &optional force)
   (let (response)
@@ -960,12 +964,14 @@ Don't cache if nil.")
       (with-current-buffer (elmo-network-session-buffer session)
 	(std11-field-body "Newsgroups")))))
 
-(luna-define-method elmo-message-fetch-with-cache-process :after
+(luna-define-method elmo-message-fetch-with-cache-process :around
   ((folder elmo-nntp-folder) number strategy &optional section unread)
-  (elmo-nntp-setup-crosspost-buffer folder number)
-  (unless unread
-    (elmo-nntp-folder-update-crosspost-message-alist
-     folder (list number))))
+  (when (luna-call-next-method)
+    (elmo-nntp-setup-crosspost-buffer folder number)
+    (unless unread
+      (elmo-nntp-folder-update-crosspost-message-alist
+       folder (list number)))
+    t))
 
 (luna-define-method elmo-message-fetch-plugged ((folder elmo-nntp-folder)
 						number strategy
@@ -1121,27 +1127,27 @@ Returns a list of cons cells like (NUMBER . VALUE)"
 	numbers))
      ((or (string= "since" search-key)
 	  (string= "before" search-key))
-      (let* ((key-date (elmo-date-get-datevec (elmo-filter-value condition)))
-	     (key-datestr (elmo-date-make-sortable-string key-date))
+      (let* ((specified-date (elmo-date-make-sortable-string
+			      (elmo-date-get-datevec (elmo-filter-value
+						      condition))))
 	     (since (string= "since" search-key))
-	     result)
+	     field-date  result)
 	(if (eq (elmo-filter-type condition) 'unmatch)
 	    (setq since (not since)))
 	(setq result
 	      (delq nil
 		    (mapcar
 		     (lambda (pair)
+		       (setq field-date
+			     (elmo-date-make-sortable-string
+			      (timezone-fix-time
+			       (cdr pair)
+			       (current-time-zone) nil)))
 		       (if (if since
-			       (string< key-datestr
-					(elmo-date-make-sortable-string
-					 (timezone-fix-time
-					  (cdr pair)
-					  (current-time-zone) nil)))
-			     (not (string< key-datestr
-					   (elmo-date-make-sortable-string
-					    (timezone-fix-time
-					     (cdr pair)
-					     (current-time-zone) nil)))))
+			       (or (string= specified-date field-date)
+				   (string< specified-date field-date))
+			     (string< field-date
+				      specified-date))
 			   (car pair)))
 		     (elmo-nntp-retrieve-field spec "date" from-msgs))))
 	(if from-msgs
