@@ -816,29 +816,26 @@ Return value is a cons cell of (STRUCTURE . REST)"
 			 (length (memq number number-list)))
 		      (string-to-int (elmo-filter-value condition)))))
      ((string= (elmo-filter-key condition) "since")
-      (let ((date (elmo-date-get-datevec (elmo-filter-value condition))))
+      (let ((field-date (elmo-date-make-sortable-string
+			 (timezone-fix-time
+			  (std11-field-body "date")
+			  (current-time-zone) nil)))
+	    (specified-date (elmo-date-make-sortable-string
+			     (elmo-date-get-datevec
+			      (elmo-filter-value condition)))))
 	(setq result
-	      (string<
-	       (timezone-make-sortable-date (aref date 0)
-					    (aref date 1)
-					    (aref date 2)
-					    (timezone-make-time-string
-					     (aref date 3)
-					     (aref date 4)
-					     (aref date 5)))
-	       (timezone-make-date-sortable (std11-field-body "date"))))))
+	      (or (string= field-date specified-date)
+		  (string< specified-date field-date)))))
      ((string= (elmo-filter-key condition) "before")
-      (let ((date (elmo-date-get-datevec (elmo-filter-value condition))))
-	(setq result
-	      (string<
-	       (timezone-make-date-sortable (std11-field-body "date"))
-	       (timezone-make-sortable-date (aref date 0)
-					    (aref date 1)
-					    (aref date 2)
-					    (timezone-make-time-string
-					     (aref date 3)
-					     (aref date 4)
-					     (aref date 5)))))))
+      (setq result
+	    (string<
+	     (elmo-date-make-sortable-string
+	      (timezone-fix-time
+	       (std11-field-body "date")
+	       (current-time-zone) nil))
+	     (elmo-date-make-sortable-string
+	      (elmo-date-get-datevec
+	       (elmo-filter-value condition))))))
      ((string= (elmo-filter-key condition) "body")
       (and (re-search-forward "^$" nil t)	   ; goto body
 	   (setq result (search-forward (elmo-filter-value condition)
@@ -897,11 +894,15 @@ Return value is a cons cell of (STRUCTURE . REST)"
     (cond
      ((string= (elmo-filter-key condition) "last")
       (setq result (<= (length (memq number number-list))
-		       (string-to-int (elmo-filter-value condition)))))
+		       (string-to-int (elmo-filter-value condition))))
+      (if (eq (elmo-filter-type condition) 'unmatch)
+	  (setq result (not result))))
      ((string= (elmo-filter-key condition) "first")
       (setq result (< (- (length number-list)
 			 (length (memq number number-list)))
-		      (string-to-int (elmo-filter-value condition)))))
+		      (string-to-int (elmo-filter-value condition))))
+      (if (eq (elmo-filter-type condition) 'unmatch)
+	  (setq result (not result))))
      (t
       (elmo-set-work-buf
        (as-binary-input-file (insert-file-contents file))
@@ -911,8 +912,6 @@ Return value is a cons cell of (STRUCTURE . REST)"
        (setq result
 	     (elmo-buffer-field-primitive-condition-match
 	      condition number number-list)))))
-    (if (eq (elmo-filter-type condition) 'unmatch)
-	(setq result (not result)))
     result))
 
 (defun elmo-file-field-condition-match (file condition number number-list)
@@ -1174,7 +1173,9 @@ the value of `foo'."
 (defun elmo-progress-clear (label)
   (let ((counter (assq label elmo-progress-counter-alist)))
     (when counter
-      (elmo-display-progress label "" 100)
+      (elmo-display-progress label
+			     (elmo-progress-counter-format counter)
+			     100)
       (setq elmo-progress-counter-alist
 	    (delq counter elmo-progress-counter-alist)))))
 
@@ -1608,6 +1609,24 @@ Return t if cache is saved successfully."
     ;; ignore error
     (error)))
 
+(defun elmo-file-cache-load (cache-path section)
+  "Load cache on PATH into the current buffer.
+Return t if cache is loaded successfully."
+  (condition-case nil
+      (let (cache-file)
+	(when (and cache-path
+		   (if (elmo-cache-path-section-p cache-path)
+		       section
+		     (null section))
+		   (setq cache-file (elmo-file-cache-expand-path
+				     cache-path
+				     section))
+		   (file-exists-p cache-file))
+	  (insert-file-contents-as-binary cache-file)
+	  t))
+    ;; igore error
+    (error)))
+
 (defun elmo-cache-path-section-p (path)
   "Return non-nil when PATH is `section' cache path."
   (file-directory-p path))
