@@ -109,6 +109,7 @@ If a folder name begins with PREFIX, use BACKEND."
 				     message-modified ; message is modified.
 				     mark-modified    ; mark is modified.
 				     process-duplicates  ; read or hide
+				     biff   ; folder for biff
 				     ))
   (luna-define-internal-accessors 'elmo-folder))
 
@@ -686,6 +687,9 @@ Return a cons cell of (NUMBER-CROSSPOSTS . NEW-MARK-ALIST).")
 (luna-define-method elmo-folder-writable-p ((folder elmo-folder))
   nil) ; default is not writable.
 
+(luna-define-method elmo-folder-delete ((folder elmo-folder))
+  (elmo-msgdb-delete-path folder))
+
 (luna-define-method elmo-folder-rename ((folder elmo-folder) new-name)
   (let* ((new-folder (elmo-make-folder new-name)))
     (unless (eq (elmo-folder-type-internal folder)
@@ -885,29 +889,35 @@ Return a cons cell of (NUMBER-CROSSPOSTS . NEW-MARK-ALIST).")
       (while numbers
 	(setq failure nil)
 	(condition-case nil
-	    (progn
-	      (elmo-message-fetch
-	       src-folder (car numbers)
-	       (if (and (not (elmo-folder-plugged-p src-folder))
-			elmo-enable-disconnected-operation
-			(setq cache (elmo-file-cache-get
-				     (elmo-message-field
-				      src-folder (car numbers)
-				      'message-id)))
-			(eq (elmo-file-cache-status cache) 'entire))
-		   (elmo-make-fetch-strategy
-		    'entire t nil (elmo-file-cache-path cache))
-		 (elmo-make-fetch-strategy 'entire t))
-	       nil (current-buffer)
-	       'unread)
-	      (unless (eq (buffer-size) 0)
-		(setq failure (not
-			       (elmo-folder-append-buffer
-				folder
-				(setq unseen (member (elmo-message-mark
-						      src-folder (car numbers))
-						     unread-marks))
-				(if same-number (car numbers)))))))
+	    (setq cache (elmo-file-cache-get
+			 (elmo-message-field src-folder
+					     (car numbers)
+					     'message-id))
+		  failure
+		  (not
+		   (and
+		    (elmo-message-fetch
+		     src-folder (car numbers)
+		     (if (elmo-folder-plugged-p src-folder)
+			 (elmo-make-fetch-strategy
+			  'entire 'maybe nil
+			  (and cache (elmo-file-cache-path cache)))
+		       (or (and elmo-enable-disconnected-operation
+				cache
+				(eq (elmo-file-cache-status cache) 'entire)
+				(elmo-make-fetch-strategy
+				 'entire t nil
+				 (elmo-file-cache-path cache)))
+			   (error "Unplugged")))
+		     nil (current-buffer)
+		     'unread)
+		    (> (buffer-size) 0)
+		    (elmo-folder-append-buffer
+		     folder
+		     (setq unseen (member (elmo-message-mark
+					   src-folder (car numbers))
+					  unread-marks))
+		     (if same-number (car numbers))))))
 	  (error (setq failure t)))
 	;; FETCH & APPEND finished
 	(unless failure
@@ -1124,7 +1134,7 @@ FIELD is a symbol of the field."
     (if (and elmo-folder-update-threshold
 	     (> len elmo-folder-update-threshold)
 	     elmo-folder-update-confirm)
-	(if (y-or-n-p (format "Too many messages(%d).  Continue? " len))
+	(if (y-or-n-p (format "Too many messages(%d).  Update all? " len))
 	    appends
 	  (setq in elmo-folder-update-threshold)
 	  (catch 'end
@@ -1154,8 +1164,7 @@ FIELD is a symbol of the field."
       (with-current-buffer outbuf
 	(erase-buffer)
 	(elmo-message-fetch-with-cache-process folder number
-					       strategy section unread)
-	t)
+					       strategy section unread))
     (with-temp-buffer
       (elmo-message-fetch-with-cache-process folder number
 					     strategy section unread)
@@ -1165,24 +1174,37 @@ FIELD is a symbol of the field."
 							   number strategy
 							   &optional
 							   section unread)
-  (let (cache-path cache-file)
-    (if (and (elmo-fetch-strategy-use-cache strategy)
-	     (setq cache-path (elmo-fetch-strategy-cache-path strategy))
-	     (setq cache-file (elmo-file-cache-expand-path
-			       cache-path
-			       section))
-	     (file-exists-p cache-file)
-	     (or (not (elmo-cache-path-section-p cache-file))
-		 (not (eq (elmo-fetch-strategy-entireness strategy) 'entire))))
-	(insert-file-contents-as-binary cache-file)
-      (elmo-message-fetch-internal folder number strategy section unread)
-      (elmo-delete-cr-buffer)
-      (when (and (> (buffer-size) 0)
-		 (elmo-fetch-strategy-save-cache strategy)
-		 (elmo-fetch-strategy-cache-path strategy))
-	(elmo-file-cache-save
-	 (elmo-fetch-strategy-cache-path strategy)
-	 section)))))
+  (let ((cache-path (elmo-fetch-strategy-cache-path strategy))
+	(method-priorities
+	 (cond ((eq (elmo-fetch-strategy-use-cache strategy) 'maybe)
+		'(entity cache))
+	       ((elmo-fetch-strategy-use-cache strategy)
+		'(cache entity))
+	       (t
+		'(entity))))
+	result err)
+    (while (and method-priorities
+		(null result))
+      (setq result
+	    (case (car method-priorities)
+	      (cache
+	       (elmo-file-cache-load cache-path section))
+	      (entity
+	       (when (and (condition-case error
+			      (elmo-message-fetch-internal folder number
+							   strategy
+							   section
+							   unread)
+			    (error (setq err error) nil))
+			  (> (buffer-size) 0))
+		 (elmo-delete-cr-buffer)
+		 (when (and (elmo-fetch-strategy-save-cache strategy)
+			    cache-path)
+		   (elmo-file-cache-save cache-path section))
+		 t)))
+	    method-priorities (cdr method-priorities)))
+    (or result
+	(and err (signal (car err) (cdr err))))))
 
 (luna-define-method elmo-folder-clear ((folder elmo-folder)
 				       &optional keep-killed)
@@ -1256,16 +1278,7 @@ If update process is interrupted, return nil."
 			      (length new-list)))
 		     (setq diff-2 (elmo-list-diff (car diff) new-list)))
 	    (elmo-msgdb-append-to-killed-list folder (car diff-2)))
-	  ;; Don't delete important marked messages.
-	  (setq delete-list
-		(if (eq (elmo-folder-type-internal folder) 'mark)
-		    (cadr diff)
-		  (elmo-delete-if
-		   (lambda (x)
-		     (and (setq mark (cadr (assq x mark-alist)))
-			  (string= mark important-mark)))
-		   ;; delete message list
-		   (cadr diff))))
+	  (setq delete-list (cadr diff))
 	  (if (or (equal diff '(nil nil))
 		  (equal diff '(nil))
 		  (and (eq (length (car diff)) 0)
