@@ -1,4 +1,4 @@
-;;; wl-thread.el -- Thread display modules for Wanderlust.
+;;; wl-thread.el --- Thread display modules for Wanderlust.
 
 ;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
 ;; Copyright (C) 1998,1999,2000 Masahiro MURATA  <muse@ba2.so-net.ne.jp>
@@ -26,10 +26,10 @@
 ;;
 
 ;;; Commentary:
-;; 
+;;
 
 ;;; Code:
-;; 
+;;
 
 (require 'wl-summary)
 (require 'wl-highlight)
@@ -37,10 +37,10 @@
 
 ;; buffer local variables.
 ;;(defvar wl-thread-top-entity '(nil t nil nil)) ; top entity
-(defvar wl-thread-tops nil)           ; top number list (number)
+(defvar wl-thread-tops nil)		; top number list (number)
 (defvar wl-thread-entities nil)
-(defvar wl-thread-entity-list nil)    ; entity list
-(defvar wl-thread-entity-hashtb nil)  ; obarray
+(defvar wl-thread-entity-list nil)	; entity list
+(defvar wl-thread-entity-hashtb nil)	; obarray
 (defvar wl-thread-indent-regexp nil)
 
 (make-variable-buffer-local 'wl-thread-entity-hashtb)
@@ -58,11 +58,11 @@
   (let (entities top-list)
     (setq entities (wl-summary-load-file-object
 		    (expand-file-name wl-thread-entity-file
-				      (elmo-msgdb-expand-path fld))))
+				      (elmo-folder-msgdb-path fld))))
     (setq top-list
 	  (wl-summary-load-file-object
 	   (expand-file-name wl-thread-entity-list-file
-			     (elmo-msgdb-expand-path fld))))
+			     (elmo-folder-msgdb-path fld))))
     (message "Resuming thread structure...")
     ;; set obarray value.
     (setq wl-thread-entity-hashtb (elmo-make-hash (* (length entities) 2)))
@@ -201,10 +201,10 @@
     (elmo-list-insert wl-summary-buffer-number-list
 		      (wl-thread-entity-get-number entity)
 		      (progn
-			(while (setq curc 
+			(while (setq curc
 				     (wl-thread-entity-get-children curp))
-			  (setq curp (wl-thread-get-entity 
-				      (nth (- (length curc) 1) 
+			  (setq curp (wl-thread-get-entity
+				      (nth (- (length curc) 1)
 					   curc))))
 			(wl-thread-entity-get-number curp)))
     (setcar (cddr to) (wl-append children
@@ -310,8 +310,8 @@ ENTITY is returned."
 (defun wl-thread-jump-to-msg (&optional number)
   (interactive)
   (let ((num (or number
-                 (string-to-int
-                  (read-from-minibuffer "Jump to Message(No.): ")))))
+		 (string-to-int
+		  (read-from-minibuffer "Jump to Message(No.): ")))))
     (wl-thread-entity-force-open (wl-thread-get-entity num))
     (wl-summary-jump-to-msg num)))
 
@@ -349,11 +349,12 @@ ENTITY is returned."
 	  (cur 0)
 	  entity)
       (while (not (eobp))
-	(unless (wl-thread-entity-get-opened
-		 (setq entity (wl-thread-get-entity
-			       (wl-summary-message-number))))
-	  (wl-thread-entity-force-open entity))
-	(wl-thread-goto-bottom-of-sub-thread)
+	(if (wl-thread-entity-get-opened
+	     (setq entity (wl-thread-get-entity
+			   (wl-summary-message-number))))
+	    (forward-line 1)
+	  (wl-thread-force-open)
+	  (wl-thread-goto-bottom-of-sub-thread))
 	(when (> len elmo-display-progress-threshold)
 	  (setq cur (1+ cur))
 	  (elmo-display-progress
@@ -367,7 +368,7 @@ ENTITY is returned."
 
 (defun wl-thread-open-all-unread ()
   (interactive)
-  (let ((mark-alist (elmo-msgdb-get-mark-alist wl-summary-buffer-msgdb))
+  (let ((mark-alist (elmo-msgdb-get-mark-alist (wl-summary-buffer-msgdb)))
 	mark)
     (while mark-alist
       (if (setq mark (nth 1 (car mark-alist)))
@@ -387,8 +388,8 @@ ENTITY is returned."
 (defsubst wl-thread-update-line-on-buffer-sub (entity msg &optional parent-msg)
   (let* ((entity (or entity (wl-thread-get-entity msg)))
 	 (parent-msg (or parent-msg (wl-thread-entity-get-parent entity)))
-	 (overview (elmo-msgdb-get-overview wl-summary-buffer-msgdb))
-	 (mark-alist (elmo-msgdb-get-mark-alist wl-summary-buffer-msgdb))
+	 (overview (elmo-msgdb-get-overview (wl-summary-buffer-msgdb)))
+	 (mark-alist (elmo-msgdb-get-mark-alist (wl-summary-buffer-msgdb)))
 	 (buffer-read-only nil)
 	 (inhibit-read-only t)
 	 overview-entity temp-mark summary-line invisible-top dest-pair)
@@ -406,13 +407,13 @@ ENTITY is returned."
 	   (t (setq temp-mark (wl-summary-get-score-mark msg))))
 	  (when (setq overview-entity
 		      (elmo-msgdb-overview-get-entity
-		       msg wl-summary-buffer-msgdb))
+		       msg (wl-summary-buffer-msgdb)))
 	    (setq summary-line
 		  (wl-summary-overview-create-summary-line
 		   msg
 		   overview-entity
 		   (elmo-msgdb-overview-get-entity
-		    parent-msg wl-summary-buffer-msgdb)
+		    parent-msg (wl-summary-buffer-msgdb))
 		   nil
 		   mark-alist
 		   (if wl-thread-insert-force-opened
@@ -428,13 +429,13 @@ ENTITY is returned."
       (if (not (setq invisible-top
 		     (wl-thread-entity-parent-invisible-p entity)))
 	  (wl-summary-update-thread
-	   (elmo-msgdb-overview-get-entity msg wl-summary-buffer-msgdb)
+	   (elmo-msgdb-overview-get-entity msg (wl-summary-buffer-msgdb))
 	   overview
 	   mark-alist
 	   entity
 	   (and parent-msg
 		(elmo-msgdb-overview-get-entity
-		 parent-msg wl-summary-buffer-msgdb)))
+		 parent-msg (wl-summary-buffer-msgdb))))
 	;; currently invisible.. update closed line.
 	(wl-thread-update-children-number invisible-top)))))
 
@@ -517,7 +518,7 @@ ENTITY is returned."
     (while msgs
       (setq children (wl-thread-entity-get-children
 		      (setq entity (wl-thread-get-entity (car msgs)))))
-      (when (elmo-msgdb-overview-get-entity (car msgs) wl-summary-buffer-msgdb)
+      (when (elmo-msgdb-overview-get-entity (car msgs) (wl-summary-buffer-msgdb))
 	(wl-append ret-val (list (car msgs)))
 	(setq children nil))
       (setq msgs (cdr msgs))
@@ -651,10 +652,10 @@ ENTITY is returned."
 					 ent entity nil))
 	      (setq insert-msgs (cdr insert-msgs))))))
       (if update
- 	  ;; modify buffer.
+	  ;; modify buffer.
 	  (while update-msgs
 	    (wl-thread-update-line-on-buffer-sub nil (pop update-msgs)))
- 	;; don't update buffer
+	;; don't update buffer
 	update-msgs)))) ; return value
 
 (defun wl-thread-insert-message (overview-entity overview mark-alist
@@ -687,7 +688,7 @@ Message is inserted to the summary buffer."
 	       mark-alist
 	       child-entity
 	       (elmo-msgdb-overview-get-entity
-		parent-msg wl-summary-buffer-msgdb))
+		parent-msg (wl-summary-buffer-msgdb)))
 	      (when parent
 		;; use thread structure.
 		(wl-thread-entity-get-nearly-older-brother
@@ -750,12 +751,12 @@ Message is inserted to the summary buffer."
 	(goto-char (match-end 0))
 	(setq beg (current-column))
 	(setq from-end (save-excursion
-			 (move-to-column (+ 1 beg wl-from-width))
+			 (move-to-column (+ 1 beg wl-summary-from-width))
 			 (point)))
 	(setq from (buffer-substring (match-end 0) from-end))
 	(delete-region (match-end 0) from-end)
 	(setq str (wl-set-string-width
-		   (1+ wl-from-width)
+		   (1+ wl-summary-from-width)
 		   (format
 		    "+%s:%s"
 		    (wl-thread-entity-get-children-num
@@ -770,7 +771,7 @@ Message is inserted to the summary buffer."
 	   (ding)
 	   (message "Error in wl-thread-update-children-number-hook."))))))))
 
-;; 
+;;
 ;; Thread oriented commands.
 ;;
 (defun wl-thread-call-region-func (func &optional arg)
@@ -788,7 +789,7 @@ Message is inserted to the summary buffer."
 
 (defun wl-thread-msg-mark-as-important (msg)
   "Set mark as important for invisible MSG. Modeline is not changed."
-  (let* ((msgdb wl-summary-buffer-msgdb)
+  (let* ((msgdb (wl-summary-buffer-msgdb))
 	 (mark-alist (elmo-msgdb-get-mark-alist msgdb))
 	 cur-mark)
     (setq cur-mark (cadr (assq msg mark-alist)))
@@ -828,7 +829,7 @@ Message is inserted to the summary buffer."
     (error
      (elmo-display-error err t)
      nil)))
-	
+
 (defun wl-thread-delete (&optional arg)
   (interactive "P")
   (wl-thread-call-region-func 'wl-summary-delete-region arg)
@@ -894,7 +895,7 @@ Message is inserted to the summary buffer."
 	     (/ (* cur 100) len)))))))
 
 (defsubst wl-thread-insert-entity-sub (indent entity parent-entity all)
-  (let ((mark-alist (elmo-msgdb-get-mark-alist wl-summary-buffer-msgdb))
+  (let ((mark-alist (elmo-msgdb-get-mark-alist (wl-summary-buffer-msgdb)))
 	msg-num
 	overview-entity
 	temp-mark
@@ -913,7 +914,7 @@ Message is inserted to the summary buffer."
 	(setq temp-mark (wl-summary-get-score-mark msg-num)))
       (setq overview-entity
 	    (elmo-msgdb-overview-get-entity
-	     (nth 0 entity) wl-summary-buffer-msgdb))
+	     (nth 0 entity) (wl-summary-buffer-msgdb)))
 ;;;   (wl-delete-all-overlays)
       (when overview-entity
 	(setq summary-line
@@ -921,7 +922,7 @@ Message is inserted to the summary buffer."
 	       msg-num
 	       overview-entity
 	       (elmo-msgdb-overview-get-entity
-		(nth 0 parent-entity) wl-summary-buffer-msgdb)
+		(nth 0 parent-entity) (wl-summary-buffer-msgdb))
 	       (1+ indent)
 	       mark-alist
 	       (if wl-thread-insert-force-opened
@@ -1034,8 +1035,8 @@ Message is inserted to the summary buffer."
 
 (defun wl-thread-get-children-msgs-uncached (msg &optional uncached-marks)
   (let ((children-msgs (wl-thread-get-children-msgs msg))
-	(mark-alist (elmo-msgdb-get-mark-alist wl-summary-buffer-msgdb))
-	(number-alist (elmo-msgdb-get-number-alist wl-summary-buffer-msgdb))
+	(mark-alist (elmo-msgdb-get-mark-alist (wl-summary-buffer-msgdb)))
+	(number-alist (elmo-msgdb-get-number-alist (wl-summary-buffer-msgdb)))
 	mark
 	uncached-list)
     (while children-msgs
@@ -1045,9 +1046,11 @@ Message is inserted to the summary buffer."
 					       mark-alist)))
 			(member mark uncached-marks))
 		   (and (not uncached-marks)
-			(null (elmo-cache-exists-p
-			       (cdr (assq (car children-msgs)
-					  number-alist)))))))
+			(null (elmo-file-cache-exists-p
+			       (elmo-message-field
+				wl-summary-buffer-elmo-folder
+				(car children-msgs)
+				'message-id))))))
 	  (wl-append uncached-list (list (car children-msgs))))
       (setq children-msgs (cdr children-msgs)))
     uncached-list))
@@ -1139,7 +1142,7 @@ Message is inserted to the summary buffer."
 	       (wl-thread-get-entity (wl-summary-message-number)))))))
       (wl-summary-set-message-modified)
       (set-buffer-modified-p nil))))
-  
+
 
 (defun wl-thread-get-depth-of-current-line ()
   (interactive)
@@ -1210,8 +1213,7 @@ Message is inserted to the summary buffer."
 	(setq thr-str
 	      (wl-thread-make-indent-string
 	       (wl-thread-get-entity (string-to-int (wl-match-buffer 1)))))
-	(if (and wl-summary-width
-		 wl-summary-indent-length-limit
+	(if (and wl-summary-indent-length-limit
 		 (< wl-summary-indent-length-limit
 		    (string-width thr-str)))
 	    (setq thr-str (wl-set-string-width
